//////////////////////////////////////////////////////////////////////
// IPF.h
//
// .IPF file format support functions
//
// (c)1997 plusjune.

// Create : JAN 08, 1996 plusjune, created
//          FEB 02, 1996 plusjune, IPF structure fixed
//          AUG 28, 1997 plusjune, modified for TPixmap class 

#include <windows.h>

#ifndef  __IPF_H__
#define  __IPF_H__

////////////////////////////////////////////////////////////////////////////////////////////// 
//
//  IPFFILEHEADER is saving file header format for .IPF file
//  diskimage of .IPF file construct with "IPFFILEHEADER + IPFINFO + Data bits"
//
typedef struct tagIPFFILEHEADER
{
  DWORD   signature;   // signature, must be 'IPF1' IPF_SIGNATURE (0x49504631)
  DWORD   fileSize;    // size of file
  DWORD   compression; // compression method (not used yet)
  DWORD   reserved1;   // not used yet ( plan to use for 32bit-CRC check )
  DWORD   reserved2;   // not used yet
} IPFFILEHEADER, *LPIPFFILEHEADER;


////////////////////////////////////////////////////////////////////////////////////////////// 
//
// IPFINFO structure : header of .IPF format
//
typedef struct tagIPF
{
  DWORD type;          // image type : IPF_GRAY, IPF_ALPHA, IPF_RGB, IPF_RGBA
  DWORD imageSize;     // size of image
  DWORD width;         // width of image
  DWORD height;        // height of image
} IPF, *LPIPF;


////////////////////////////////////////////////////////////////////////////////////////////// 
// Constant definitions

#ifndef MAKEFOURCC
#define MAKEFOURCC(ch0, ch1, ch2, ch3)  \  
    ((DWORD)(BYTE)(ch0) | ((DWORD)(BYTE)(ch1) << 8) |  \ 
    ((DWORD)(BYTE)(ch2) << 16) | ((DWORD)(BYTE)(ch3) << 24 )) 
#endif // MAKEFOURCC

// File signature
//
// for IPFFILEHEADER.signature
// 
#define IPF_SIGNATURE         MAKEFOURCC('I', 'P', 'F', '1')


//  Color Model definitions
//
//  for IPFHEADER.type
//
#define IPF_GRAY       1          // 256 Gray level model
#define IPF_ALPHA      2          // alpha channel (mask filter) : same as Gray
#define IPF_RGB        3          // Red, Green, Blue 
#define IPF_RGBA       4          // Red, Green, Blue, Alpha 

// Compression methods (not implemented yet)
//
// for IPFHEADER.compress
//
#define IPF_NOT_COMPRESS        0  // no compress
#define IPF_RLE_COMPRESS        1  // RLE compress
#define IPF_LZW_COMPRESS        2  // LZW compress


#endif // ! _IPF_H_

