#include "TPixmap.h"
#include "dib.h"

// helper macros
//
#define DIB_WIDTH_BYTES(bits)   (((bits) + 31) / 32 * 4)

#define RECTWIDTH(r) ((r).right - (r).left + 1)
#define RECTHEIGHT(r) ((r).bottom - (r).top + 1)

//////////////////////////////////////////////////////////////////////////////////
// constructor

TPixmap::TPixmap()
{
	m_lpNGF = (LPNGF)NULL;
}

TPixmap::~TPixmap()
{
	Destroy();
}

//////////////////////////////////////////////////////////////////////////////////
//
// Create new NGF Blank Image (NGF_RGB)
//
// int dwWidth;     width of image
// int dwHeight;    height of image
// int wType;      type of image : NGF_RGB, NGF_INDEXED
//
BOOL TPixmap::Create( DWORD dwWidth, DWORD dwHeight )
{
	// delete previous pixmap
	//
	Destroy();

	LPNGF	lpNewNGF;
	int		nImageSize;

	nImageSize = dwWidth * dwHeight * 3;
	lpNewNGF = (LPNGF)malloc( sizeof(NGF) + nImageSize );
	if( !lpNewNGF )
		return FALSE;

	lpNewNGF->abyID[0]   = 'N';
	lpNewNGF->abyID[1]   = 'G';
	lpNewNGF->abyID[2]   = 'F';
	lpNewNGF->abyID[3]   = '1';
	lpNewNGF->dwWidth     = dwWidth;
	lpNewNGF->dwHeight    = dwHeight;
	
	m_lpNGF = lpNewNGF;

	return TRUE;
}

// RGB true color BMP only
BOOL TPixmap::CreateFromBMP( LPCTSTR lpszFile )
{
	BOOL	b;

	LPBITMAPINFOHEADER lpbi = (LPBITMAPINFOHEADER)dibLoad( lpszFile );

	if( !lpbi )
		return FALSE;

	b = CreateFromDIB( lpbi );
	free( lpbi );

	return b;
}

BOOL TPixmap::CreateFromDIB( LPBITMAPINFOHEADER lpbi )
{
	// delete previous pixmap
	//
	Destroy();

	int		nImageSize;
	DWORD	x, y;
	DWORD	dwWidth;
	DWORD	dwHeight;
	LPNGF	lpNewNGF;
	LPBYTE	lpNGFData;
	LPBYTE	lpDIBData;
	LPBYTE	lpDst;
	LPBYTE	lpSrc;

	dwWidth  = (DWORD)lpbi->biWidth;
	dwHeight = (DWORD)lpbi->biHeight;

	nImageSize = dwWidth * dwHeight * 3;
	lpNewNGF = (LPNGF)malloc( sizeof( NGF ) + nImageSize );
	if( !lpNewNGF )
		return FALSE;

	lpNewNGF->abyID[0]   = 'N';
	lpNewNGF->abyID[1]   = 'G';
	lpNewNGF->abyID[2]   = 'F';
	lpNewNGF->abyID[3]   = '1';
	lpNewNGF->dwWidth     = dwWidth;
	lpNewNGF->dwHeight    = dwHeight;
	
	m_lpNGF = lpNewNGF;

	lpNGFData = LPBYTE(lpNewNGF) + sizeof( NGF );

	if( lpbi->biBitCount == 8 )
	{
		RGBQUAD		pal[256];

		memcpy( pal, LPBYTE(lpbi) + sizeof( BITMAPINFOHEADER ), sizeof( RGBQUAD ) * 256 );
		lpDIBData = LPBYTE(lpbi) + sizeof( BITMAPINFOHEADER ) + sizeof( RGBQUAD ) * 256;

		for( y = 0 ; y < dwHeight ; y++ )
		{
			lpDst = lpNGFData + y * dwWidth * 3;
			lpSrc = lpDIBData + ( ( dwHeight - 1 ) - y ) * dwWidth;
			for( x = 0 ; x < dwWidth ; x++ )
			{
				*( lpDst + 0 ) = pal[*lpSrc].rgbRed;
				*( lpDst + 1 ) = pal[*lpSrc].rgbGreen;
				*( lpDst + 2 ) = pal[*lpSrc].rgbBlue;
				lpDst += 3;
				lpSrc++;
			}
		}
	}
	else
	if( lpbi->biBitCount == 24 )
	{
		lpDIBData = LPBYTE(lpbi) + sizeof( BITMAPINFOHEADER );

		for( y = 0 ; y < dwHeight ; y++ )
		{
			lpDst = lpNGFData + y * dwWidth * 3;
			lpSrc = lpDIBData + ( ( dwHeight - 1 ) - y ) * dwWidth * 3;
			for( x = 0 ; x < dwWidth ; x++ )
			{
				*( lpDst + 0 ) = *( lpSrc + 2 );
				*( lpDst + 1 ) = *( lpSrc + 1 );
				*( lpDst + 2 ) = *( lpSrc + 0 );
				lpDst += 3;
				lpSrc += 3;
			}
		}
	}

	return TRUE;
}

int TPixmap::GetWidth() 
{
	if( !m_lpNGF )
		return 0;

	return (int)m_lpNGF->dwWidth;
}

int TPixmap::GetHeight() 
{
	if( !m_lpNGF )
		return 0;

	return (int)m_lpNGF->dwHeight;
}

LPBYTE TPixmap::GetData()
{
	if( !m_lpNGF )
		return 0;

	return ( LPBYTE(m_lpNGF) + sizeof(NGF) );
}

int TPixmap::GetWidthBytes() 
{
	if( !m_lpNGF )
		return 0;

	return m_lpNGF->dwWidth * 3;
}

int TPixmap::GetHeightBytes() 
{
	if( !m_lpNGF )
		return 0;
	
	return m_lpNGF->dwHeight * 3;
}

void TPixmap::GetHorzScanLine(LPBYTE lpScanBuff, int nScanLine)
{
	if( !m_lpNGF )
		return;

	int nWidthBytes = GetWidthBytes();
	LPBYTE lpData = GetData() + nScanLine * nWidthBytes;

	memcpy( lpScanBuff, lpData, nWidthBytes );
}

void TPixmap::SetHorzScanLine(LPBYTE lpScanBuff, int nScanLine)
{
	if( !m_lpNGF )
		return;

	int nWidthBytes = GetWidthBytes();
	LPBYTE lpData = GetData() + nScanLine * nWidthBytes;

	memcpy( lpData, lpScanBuff, nWidthBytes );
}


void TPixmap::GetVertScanLine(LPBYTE lpScanBuff, int nColum)
{
	if( !m_lpNGF )
		return;

	int nWidthBytes = GetWidthBytes();
	LPBYTE lpData;

	for( int nScanline = 0; nScanline < (int)m_lpNGF->dwHeight; nScanline++ )
	{
		lpData = GetData() + nScanline * nWidthBytes + nColum * 3;
		*lpScanBuff++ = *lpData++;
		*lpScanBuff++ = *lpData++;
		*lpScanBuff++ = *lpData++;
	}
}

void TPixmap::SetVertScanLine(LPBYTE lpScanBuff, int nColum)
{
	if( !m_lpNGF )
		return;

	int nWidthBytes = GetWidthBytes();
	LPBYTE lpData;

	for( int nScanline = 0; nScanline < (int)m_lpNGF->dwHeight; nScanline++ )
	{
		lpData = GetData() + nScanline * nWidthBytes + nColum * 3;
		*lpData++ = *lpScanBuff++;
		*lpData++ = *lpScanBuff++;
		*lpData++ = *lpScanBuff++;
	}
}

void TPixmap::Clear()
{
	if( m_lpNGF )
		ZeroMemory( GetData(), GetWidthBytes() * m_lpNGF->dwHeight );
}

void TPixmap::Destroy()
{
	if( m_lpNGF )
	{
		free( m_lpNGF );
		m_lpNGF = (LPNGF)NULL;
	}
}

HBITMAP TPixmap::ClipRectToBitmap( HDC hdc, LPRECT lpRect )
{
	int startx, starty;  // left, top of lpRect
	int cx, cy;          // width and height

	if( lpRect )
	{
		cx = RECTWIDTH(*lpRect);
		cy = RECTHEIGHT(*lpRect);

		cx = cx <= (int)m_lpNGF->dwWidth ? cx : (int)m_lpNGF->dwWidth;
		cy = cy <= (int)m_lpNGF->dwHeight ? cy : (int)m_lpNGF->dwHeight;

		startx = lpRect->left;
		starty = lpRect->top;
	}
	else // lpRect == NULL
	{
		cx = m_lpNGF->dwWidth;
		cy = m_lpNGF->dwHeight;
		
		startx = 0;
		starty = 0;
	}

	// Create bitmap buffer and get dwWidth bytes of DDB and pointer to databits
	//
	int nWidthBytesDDB = DIB_WIDTH_BYTES(cx * 24);  // bytes pre line for RGB DDB.
	LPBYTE lpDataBitsDDB = (LPBYTE)malloc( nWidthBytesDDB * cy );

	if(!lpDataBitsDDB)
		return NULL;

	// Get dwWidth bytes of NGF and pointer to databits
	//
	int nWidthBytesNGF =  GetWidthBytes();
	LPBYTE lpDataBitsNGF = (LPBYTE)m_lpNGF + sizeof(NGF);

	// copy NGF bitmap to DDB bitmap 
	//
	{
		int x, y;
		LPBYTE lpScanlineDDB; 
		LPBYTE lpScanlineNGF; 

		for( y = 0; y < cy; y++ )
		{
			// get y scanline of DDB and NGF
			//
			lpScanlineDDB = lpDataBitsDDB + nWidthBytesDDB * ( cy - (y+1) );
			lpScanlineNGF = lpDataBitsNGF + nWidthBytesNGF * (y + starty) + startx * 3;
			
			for( x = 0; x < cx; x++ )
			{
				lpScanlineDDB[2] = *lpScanlineNGF++;
				lpScanlineDDB[1] = *lpScanlineNGF++;
				lpScanlineDDB[0] = *lpScanlineNGF++;
				lpScanlineDDB += 3;
			}
		}
	}

	// Create BITMAP Header 
	//
	LPBITMAPINFO lpbm = (LPBITMAPINFO)malloc( sizeof( BITMAPINFO ) );
	if(!lpbm)
	{
		free( lpDataBitsDDB );
		return NULL;
	}

	lpbm->bmiHeader.biSize = sizeof(BITMAPINFOHEADER);
	lpbm->bmiHeader.biWidth  = cx;
	lpbm->bmiHeader.biHeight = cy;
	lpbm->bmiHeader.biSizeImage = nWidthBytesDDB * cy;  // (DWORD)cx * cy * 3;
	lpbm->bmiHeader.biCompression = BI_RGB;
	lpbm->bmiHeader.biBitCount = 24;
	lpbm->bmiHeader.biPlanes = 1;
	lpbm->bmiHeader.biClrUsed = 0;
	lpbm->bmiHeader.biClrImportant = 0;
	lpbm->bmiHeader.biXPelsPerMeter = 0;
	lpbm->bmiHeader.biYPelsPerMeter = 0;        

	HBITMAP hBitmap = CreateCompatibleBitmap( hdc, cx, cy );

    SetDIBits( hdc,          // DDBȣȯ hDC
			   hBitmap,                    // DDBڵ
			   0,                          // lpBits  ĵ 
			   cy,                         // lpBits ĵμ
			   lpDataBitsDDB,              // Ʈ鿡  
			   lpbm,                       // DDB   
			   DIB_RGB_COLORS);            // DDB RGB

	free( lpbm );
	free( lpDataBitsDDB );

	return hBitmap;
}

void TPixmap::Draw(HANDLE h, int x, int y)
{
	HDC hdc = HDC( h );

	if(!m_lpNGF)
		return;

	RECT r;
	
	r.left = x;
	r.top = y;
	r.right = GetWidth() + x - 1;
	r.bottom = GetHeight() + y - 1;

	Draw(hdc, &r, NULL);

}

void TPixmap::Draw(HANDLE h, LPRECT lpRectDC, LPRECT lpRectNGF)
{
	HDC hdc = HDC( h );
	HBITMAP	hOld;

	if(!m_lpNGF)
		return;

	// set DC RECT and NGF RECT
	RECT rectDC;

	if( lpRectDC )
		rectDC = *lpRectDC;
	else 
		SetRect( &rectDC, 0, 0, m_lpNGF->dwWidth - 1, m_lpNGF->dwHeight - 1 );

	RECT rectNGF;

	if( lpRectNGF )
		rectNGF = *lpRectNGF;
	else 
		SetRect( &rectNGF, 0, 0, m_lpNGF->dwWidth - 1, m_lpNGF->dwHeight - 1 );

	HBITMAP hBitmap = ClipRectToBitmap( hdc, &rectNGF );
	if( !hBitmap )
		return;

	HDC hdcMem = CreateCompatibleDC( hdc );
	if( !hdcMem )
	{
		DeleteObject(hBitmap);
		return;
	}

	hOld = (HBITMAP)SelectObject( hdcMem, hBitmap );

	if( (RECTWIDTH( rectDC )  == RECTWIDTH( rectNGF )) &&
		(RECTHEIGHT( rectDC ) == RECTHEIGHT( rectNGF )) )
	{
		BitBlt( hdc,
				rectDC.left,
				rectDC.top,
				RECTWIDTH( rectDC ),
				RECTHEIGHT( rectDC ),
				hdcMem,
				rectNGF.left,
				rectNGF.top,
				SRCCOPY );
	}
	else
	{
		SetStretchBltMode( hdc, COLORONCOLOR );
		StretchBlt( hdc,
					rectDC.left,
					rectDC.top,
					RECTWIDTH( rectDC ),
					RECTHEIGHT( rectDC ),
					hdcMem,
					rectNGF.left,
					rectNGF.top,
					RECTWIDTH( rectNGF ),
					RECTHEIGHT( rectNGF ),
					SRCCOPY );
	}

	SelectObject( hdcMem, hOld );
	DeleteObject( hBitmap );
	DeleteDC( hdcMem );
}

